@@ -2,29 +2,35 @@
  * Oracle
  */
 
-/* \summary: Oracle DLT_PPI printer */
+/* \summary: Per-Packet Information (DLT_PPI) printer */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+/* Specification:
+ * Per-Packet Information Header Specification - Version 1.0.7
+ * https://web.archive.org/web/20160328114748/http://www.cacetech.com/documents/PPI%20Header%20format%201.0.7.pdf
+ */
 
-#include <netdissect-stdinc.h>
+#include <config.h>
+
+#include "netdissect-stdinc.h"
 
 #include "netdissect.h"
 #include "extract.h"
 
+
 typedef struct ppi_header {
-	uint8_t		ppi_ver;
-	uint8_t		ppi_flags;
-	uint16_t	ppi_len;
-	uint32_t	ppi_dlt;
+	nd_uint8_t	ppi_ver;	/* Version.  Currently 0 */
+	nd_uint8_t	ppi_flags;	/* Flags. */
+	nd_uint16_t	ppi_len;	/* Length of entire message, including
+					 * this header and TLV payload. */
+	nd_uint32_t	ppi_dlt;	/* Data Link Type of the captured
+					 * packet data. */
 } ppi_header_t;
 
 #define	PPI_HDRLEN	8
 
 #ifdef DLT_PPI
 
-static inline void
+static void
 ppi_header_print(netdissect_options *ndo, const u_char *bp, u_int length)
 {
 	const ppi_header_t *hdr;
@@ -34,24 +40,30 @@ ppi_header_print(netdissect_options *ndo, const u_char *bp, u_int length)
 
 	hdr = (const ppi_header_t *)bp;
 
-	len = EXTRACT_LE_16BITS(&hdr->ppi_len);
-	dlt = EXTRACT_LE_32BITS(&hdr->ppi_dlt);
+	len = GET_LE_U_2(hdr->ppi_len);
+	dlt = GET_LE_U_4(hdr->ppi_dlt);
 	dltname = pcap_datalink_val_to_name(dlt);
 
 	if (!ndo->ndo_qflag) {
-		ND_PRINT((ndo, "V.%d DLT %s (%d) len %d", hdr->ppi_ver,
+		ND_PRINT("V.%u DLT %s (%u) len %u", GET_U_1(hdr->ppi_ver),
 			  (dltname != NULL ? dltname : "UNKNOWN"), dlt,
-                          len));
+                          len);
         } else {
-		ND_PRINT((ndo, "%s", (dltname != NULL ? dltname : "UNKNOWN")));
+		ND_PRINT("%s", (dltname != NULL ? dltname : "UNKNOWN"));
         }
 
-	ND_PRINT((ndo, ", length %u: ", length));
+	ND_PRINT(", length %u: ", length);
 }
 
-static u_int
-ppi_print(netdissect_options *ndo,
-               const struct pcap_pkthdr *h, const u_char *p)
+/*
+ * This is the top level routine of the printer.  'p' points
+ * to the ether header of the packet, 'h->ts' is the timestamp,
+ * 'h->len' is the length of the packet off the wire, and 'h->caplen'
+ * is the number of bytes actually captured.
+ */
+void
+ppi_if_print(netdissect_options *ndo,
+	     const struct pcap_pkthdr *h, const u_char *p)
 {
 	if_printer printer;
 	const ppi_header_t *hdr;
@@ -62,28 +74,33 @@ ppi_print(netdissect_options *ndo,
 	uint32_t hdrlen;
 	struct pcap_pkthdr nhdr;
 
+	ndo->ndo_protocol = "ppi";
 	if (caplen < sizeof(ppi_header_t)) {
-		ND_PRINT((ndo, "[|ppi]"));
-		return (caplen);
+		nd_print_trunc(ndo);
+		ndo->ndo_ll_hdr_len += caplen;
+		return;
 	}
 
 	hdr = (const ppi_header_t *)p;
-	ND_TCHECK_16BITS(&hdr->ppi_len);
-	len = EXTRACT_LE_16BITS(&hdr->ppi_len);
+	len = GET_LE_U_2(hdr->ppi_len);
+	if (len < sizeof(ppi_header_t) || len > 65532) {
+		/* It MUST be between 8 and 65,532 inclusive (spec 3.1.3) */
+		ND_PRINT(" [length %u < %zu or > 65532]", len,
+			 sizeof(ppi_header_t));
+		nd_print_invalid(ndo);
+		ndo->ndo_ll_hdr_len += caplen;
+		return;
+	}
 	if (caplen < len) {
 		/*
 		 * If we don't have the entire PPI header, don't
 		 * bother.
 		 */
-		ND_PRINT((ndo, "[|ppi]"));
-		return (caplen);
+		nd_print_trunc(ndo);
+		ndo->ndo_ll_hdr_len += caplen;
+		return;
 	}
-	if (len < sizeof(ppi_header_t)) {
-		ND_PRINT((ndo, "[|ppi]"));
-		return (len);
-	}
-	ND_TCHECK_32BITS(&hdr->ppi_dlt);
-	dlt = EXTRACT_LE_32BITS(&hdr->ppi_dlt);
+	dlt = GET_LE_U_4(hdr->ppi_dlt);
 
 	if (ndo->ndo_eflag)
 		ppi_header_print(ndo, p, length);
@@ -92,11 +109,13 @@ ppi_print(netdissect_options *ndo,
 	caplen -= len;
 	p += len;
 
-	if ((printer = lookup_printer(dlt)) != NULL) {
+	printer = lookup_printer(dlt);
+	if (printer != NULL) {
 		nhdr = *h;
 		nhdr.caplen = caplen;
 		nhdr.len = length;
-		hdrlen = printer(ndo, &nhdr, p);
+		printer(ndo, &nhdr, p);
+		hdrlen = ndo->ndo_ll_hdr_len;
 	} else {
 		if (!ndo->ndo_eflag)
 			ppi_header_print(ndo, (const u_char *)hdr, length + len);
@@ -105,29 +124,6 @@ ppi_print(netdissect_options *ndo,
 			ND_DEFAULTPRINT(p, caplen);
 		hdrlen = 0;
 	}
-	return (len + hdrlen);
-trunc:
-	return (caplen);
+	ndo->ndo_ll_hdr_len += len + hdrlen;
 }
-
-/*
- * This is the top level routine of the printer.  'p' points
- * to the ether header of the packet, 'h->ts' is the timestamp,
- * 'h->len' is the length of the packet off the wire, and 'h->caplen'
- * is the number of bytes actually captured.
- */
-u_int
-ppi_if_print(netdissect_options *ndo,
-               const struct pcap_pkthdr *h, const u_char *p)
-{
-	return (ppi_print(ndo, h, p));
-}
-
-/*
- * Local Variables:
- * c-style: whitesmith
- * c-basic-offset: 8
- * End:
- */
-
 #endif /* DLT_PPI */
