@@ -19,14 +19,13 @@
 
 /* \summary: sFlow protocol printer */
 
-/* specification: http://www.sflow.org/developers/specifications.php */
+/* specification: https://sflow.org/developers/specifications.php */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include <config.h>
 
-#include <netdissect-stdinc.h>
+#include "netdissect-stdinc.h"
 
+#define ND_LONGJMP_FROM_TCHECK
 #include "netdissect.h"
 #include "extract.h"
 #include "addrtoname.h"
@@ -55,18 +54,28 @@
  */
 
 struct sflow_datagram_t {
-    uint8_t 	version[4];
-    uint8_t 	ip_version[4];
-    uint8_t 	agent[4];
-    uint8_t 	agent_id[4];
-    uint8_t 	seqnum[4];
-    uint8_t 	uptime[4];
-    uint8_t 	samples[4];
+    nd_uint32_t version;
+    nd_uint32_t ip_version;
+    nd_ipv4	agent;
+    nd_uint32_t	agent_id;
+    nd_uint32_t	seqnum;
+    nd_uint32_t	uptime;
+    nd_uint32_t	samples;
+};
+
+struct sflow_v6_datagram_t {
+    nd_uint32_t version;
+    nd_uint32_t ip_version;
+    nd_ipv6     agent;
+    nd_uint32_t	agent_id;
+    nd_uint32_t	seqnum;
+    nd_uint32_t	uptime;
+    nd_uint32_t	samples;
 };
 
 struct sflow_sample_header {
-    uint8_t	format[4];
-    uint8_t	len[4];
+    nd_uint32_t	format;
+    nd_uint32_t	len;
 };
 
 #define		SFLOW_FLOW_SAMPLE		1
@@ -83,47 +92,48 @@ static const struct tok sflow_format_values[] = {
 };
 
 struct sflow_flow_sample_t {
-    uint8_t    seqnum[4];
-    uint8_t    typesource[4];
-    uint8_t    rate[4];
-    uint8_t    pool[4];
-    uint8_t    drops[4];
-    uint8_t    in_interface[4];
-    uint8_t    out_interface[4];
-    uint8_t    records[4];
+    nd_uint32_t seqnum;
+    nd_uint8_t  type;
+    nd_uint24_t index;
+    nd_uint32_t rate;
+    nd_uint32_t pool;
+    nd_uint32_t drops;
+    nd_uint32_t in_interface;
+    nd_uint32_t out_interface;
+    nd_uint32_t records;
 
 };
 
 struct sflow_expanded_flow_sample_t {
-    uint8_t    seqnum[4];
-    uint8_t    type[4];
-    uint8_t    index[4];
-    uint8_t    rate[4];
-    uint8_t    pool[4];
-    uint8_t    drops[4];
-    uint8_t    in_interface_format[4];
-    uint8_t    in_interface_value[4];
-    uint8_t    out_interface_format[4];
-    uint8_t    out_interface_value[4];
-    uint8_t    records[4];
+    nd_uint32_t seqnum;
+    nd_uint32_t type;
+    nd_uint32_t index;
+    nd_uint32_t rate;
+    nd_uint32_t pool;
+    nd_uint32_t drops;
+    nd_uint32_t in_interface_format;
+    nd_uint32_t in_interface_value;
+    nd_uint32_t out_interface_format;
+    nd_uint32_t out_interface_value;
+    nd_uint32_t records;
 };
 
-#define 	SFLOW_FLOW_RAW_PACKET			1
-#define 	SFLOW_FLOW_ETHERNET_FRAME		2
-#define 	SFLOW_FLOW_IPV4_DATA			3
-#define 	SFLOW_FLOW_IPV6_DATA			4
-#define 	SFLOW_FLOW_EXTENDED_SWITCH_DATA		1001
-#define 	SFLOW_FLOW_EXTENDED_ROUTER_DATA		1002
-#define 	SFLOW_FLOW_EXTENDED_GATEWAY_DATA 	1003
-#define 	SFLOW_FLOW_EXTENDED_USER_DATA		1004
-#define 	SFLOW_FLOW_EXTENDED_URL_DATA		1005
-#define 	SFLOW_FLOW_EXTENDED_MPLS_DATA		1006
-#define 	SFLOW_FLOW_EXTENDED_NAT_DATA		1007
-#define 	SFLOW_FLOW_EXTENDED_MPLS_TUNNEL		1008
-#define 	SFLOW_FLOW_EXTENDED_MPLS_VC		1009
-#define 	SFLOW_FLOW_EXTENDED_MPLS_FEC		1010
-#define 	SFLOW_FLOW_EXTENDED_MPLS_LVP_FEC	1011
-#define 	SFLOW_FLOW_EXTENDED_VLAN_TUNNEL		1012
+#define	SFLOW_FLOW_RAW_PACKET			1
+#define	SFLOW_FLOW_ETHERNET_FRAME		2
+#define	SFLOW_FLOW_IPV4_DATA			3
+#define	SFLOW_FLOW_IPV6_DATA			4
+#define	SFLOW_FLOW_EXTENDED_SWITCH_DATA		1001
+#define	SFLOW_FLOW_EXTENDED_ROUTER_DATA		1002
+#define	SFLOW_FLOW_EXTENDED_GATEWAY_DATA	1003
+#define	SFLOW_FLOW_EXTENDED_USER_DATA		1004
+#define	SFLOW_FLOW_EXTENDED_URL_DATA		1005
+#define	SFLOW_FLOW_EXTENDED_MPLS_DATA		1006
+#define	SFLOW_FLOW_EXTENDED_NAT_DATA		1007
+#define	SFLOW_FLOW_EXTENDED_MPLS_TUNNEL		1008
+#define	SFLOW_FLOW_EXTENDED_MPLS_VC		1009
+#define	SFLOW_FLOW_EXTENDED_MPLS_FEC		1010
+#define	SFLOW_FLOW_EXTENDED_MPLS_LVP_FEC	1011
+#define	SFLOW_FLOW_EXTENDED_VLAN_TUNNEL		1012
 
 static const struct tok sflow_flow_type_values[] = {
     { SFLOW_FLOW_RAW_PACKET, "Raw packet"},
@@ -157,47 +167,48 @@ static const struct tok sflow_flow_raw_protocol_values[] = {
 };
 
 struct sflow_expanded_flow_raw_t {
-    uint8_t    protocol[4];
-    uint8_t    length[4];
-    uint8_t    stripped_bytes[4];
-    uint8_t    header_size[4];
+    nd_uint32_t protocol;
+    nd_uint32_t length;
+    nd_uint32_t stripped_bytes;
+    nd_uint32_t header_size;
 };
 
 struct sflow_ethernet_frame_t {
-    uint8_t length[4];
-    uint8_t src_mac[8];
-    uint8_t dst_mac[8];
-    uint8_t type[4];
+    nd_uint32_t length;
+    nd_byte     src_mac[8];
+    nd_byte     dst_mac[8];
+    nd_uint32_t type;
 };
 
 struct sflow_extended_switch_data_t {
-    uint8_t src_vlan[4];
-    uint8_t src_pri[4];
-    uint8_t dst_vlan[4];
-    uint8_t dst_pri[4];
+    nd_uint32_t src_vlan;
+    nd_uint32_t src_pri;
+    nd_uint32_t dst_vlan;
+    nd_uint32_t dst_pri;
 };
 
 struct sflow_counter_record_t {
-    uint8_t    format[4];
-    uint8_t    length[4];
+    nd_uint32_t    format;
+    nd_uint32_t    length;
 };
 
 struct sflow_flow_record_t {
-    uint8_t    format[4];
-    uint8_t    length[4];
+    nd_uint32_t    format;
+    nd_uint32_t    length;
 };
 
 struct sflow_counter_sample_t {
-    uint8_t    seqnum[4];
-    uint8_t    typesource[4];
-    uint8_t    records[4];
+    nd_uint32_t    seqnum;
+    nd_uint8_t     type;
+    nd_uint24_t    index;
+    nd_uint32_t    records;
 };
 
 struct sflow_expanded_counter_sample_t {
-    uint8_t    seqnum[4];
-    uint8_t    type[4];
-    uint8_t    index[4];
-    uint8_t    records[4];
+    nd_uint32_t    seqnum;
+    nd_uint32_t    type;
+    nd_uint32_t    index;
+    nd_uint32_t    records;
 };
 
 #define         SFLOW_COUNTER_GENERIC           1
@@ -233,67 +244,67 @@ static const struct tok sflow_iface_direction_values[] = {
 };
 
 struct sflow_generic_counter_t {
-    uint8_t    ifindex[4];
-    uint8_t    iftype[4];
-    uint8_t    ifspeed[8];
-    uint8_t    ifdirection[4];
-    uint8_t    ifstatus[4];
-    uint8_t    ifinoctets[8];
-    uint8_t    ifinunicastpkts[4];
-    uint8_t    ifinmulticastpkts[4];
-    uint8_t    ifinbroadcastpkts[4];
-    uint8_t    ifindiscards[4];
-    uint8_t    ifinerrors[4];
-    uint8_t    ifinunkownprotos[4];
-    uint8_t    ifoutoctets[8];
-    uint8_t    ifoutunicastpkts[4];
-    uint8_t    ifoutmulticastpkts[4];
-    uint8_t    ifoutbroadcastpkts[4];
-    uint8_t    ifoutdiscards[4];
-    uint8_t    ifouterrors[4];
-    uint8_t    ifpromiscmode[4];
+    nd_uint32_t    ifindex;
+    nd_uint32_t    iftype;
+    nd_uint64_t    ifspeed;
+    nd_uint32_t    ifdirection;
+    nd_uint32_t    ifstatus;
+    nd_uint64_t    ifinoctets;
+    nd_uint32_t    ifinunicastpkts;
+    nd_uint32_t    ifinmulticastpkts;
+    nd_uint32_t    ifinbroadcastpkts;
+    nd_uint32_t    ifindiscards;
+    nd_uint32_t    ifinerrors;
+    nd_uint32_t    ifinunkownprotos;
+    nd_uint64_t    ifoutoctets;
+    nd_uint32_t    ifoutunicastpkts;
+    nd_uint32_t    ifoutmulticastpkts;
+    nd_uint32_t    ifoutbroadcastpkts;
+    nd_uint32_t    ifoutdiscards;
+    nd_uint32_t    ifouterrors;
+    nd_uint32_t    ifpromiscmode;
 };
 
 struct sflow_ethernet_counter_t {
-    uint8_t    alignerrors[4];
-    uint8_t    fcserrors[4];
-    uint8_t    single_collision_frames[4];
-    uint8_t    multiple_collision_frames[4];
-    uint8_t    test_errors[4];
-    uint8_t    deferred_transmissions[4];
-    uint8_t    late_collisions[4];
-    uint8_t    excessive_collisions[4];
-    uint8_t    mac_transmit_errors[4];
-    uint8_t    carrier_sense_errors[4];
-    uint8_t    frame_too_longs[4];
-    uint8_t    mac_receive_errors[4];
-    uint8_t    symbol_errors[4];
+    nd_uint32_t    alignerrors;
+    nd_uint32_t    fcserrors;
+    nd_uint32_t    single_collision_frames;
+    nd_uint32_t    multiple_collision_frames;
+    nd_uint32_t    test_errors;
+    nd_uint32_t    deferred_transmissions;
+    nd_uint32_t    late_collisions;
+    nd_uint32_t    excessive_collisions;
+    nd_uint32_t    mac_transmit_errors;
+    nd_uint32_t    carrier_sense_errors;
+    nd_uint32_t    frame_too_longs;
+    nd_uint32_t    mac_receive_errors;
+    nd_uint32_t    symbol_errors;
 };
 
 struct sflow_100basevg_counter_t {
-    uint8_t    in_highpriority_frames[4];
-    uint8_t    in_highpriority_octets[8];
-    uint8_t    in_normpriority_frames[4];
-    uint8_t    in_normpriority_octets[8];
-    uint8_t    in_ipmerrors[4];
-    uint8_t    in_oversized[4];
-    uint8_t    in_data_errors[4];
-    uint8_t    in_null_addressed_frames[4];
-    uint8_t    out_highpriority_frames[4];
-    uint8_t    out_highpriority_octets[8];
-    uint8_t    transitioninto_frames[4];
-    uint8_t    hc_in_highpriority_octets[8];
-    uint8_t    hc_in_normpriority_octets[8];
-    uint8_t    hc_out_highpriority_octets[8];
+    nd_uint32_t    in_highpriority_frames;
+    nd_uint64_t    in_highpriority_octets;
+    nd_uint32_t    in_normpriority_frames;
+    nd_uint64_t    in_normpriority_octets;
+    nd_uint32_t    in_ipmerrors;
+    nd_uint32_t    in_oversized;
+    nd_uint32_t    in_data_errors;
+    nd_uint32_t    in_null_addressed_frames;
+    nd_uint32_t    out_highpriority_frames;
+    nd_uint64_t    out_highpriority_octets;
+    nd_uint32_t    transitioninto_frames;
+    nd_uint64_t    hc_in_highpriority_octets;
+    nd_uint64_t    hc_in_normpriority_octets;
+    nd_uint64_t    hc_out_highpriority_octets;
 };
 
 struct sflow_vlan_counter_t {
-    uint8_t    vlan_id[4];
-    uint8_t    octets[8];
-    uint8_t    unicast_pkt[4];
-    uint8_t    multicast_pkt[4];
-    uint8_t    broadcast_pkt[4];
-    uint8_t    discards[4];
+    nd_uint32_t    vlan_id;
+    nd_uint64_t    octets;
+    nd_uint32_t    unicast_pkt;
+    nd_uint32_t    multicast_pkt;
+    nd_uint32_t    broadcast_pkt;
+    nd_uint32_t    discards;
 };
 
 static int
@@ -306,43 +317,39 @@ print_sflow_counter_generic(netdissect_options *ndo,
 	return 1;
 
     sflow_gen_counter = (const struct sflow_generic_counter_t *)pointer;
-    ND_TCHECK(*sflow_gen_counter);
-    ND_PRINT((ndo, "\n\t      ifindex %u, iftype %u, ifspeed %" PRIu64 ", ifdirection %u (%s)",
-	   EXTRACT_32BITS(sflow_gen_counter->ifindex),
-	   EXTRACT_32BITS(sflow_gen_counter->iftype),
-	   EXTRACT_64BITS(sflow_gen_counter->ifspeed),
-	   EXTRACT_32BITS(sflow_gen_counter->ifdirection),
+    ND_PRINT("\n\t      ifindex %u, iftype %u, ifspeed %" PRIu64 ", ifdirection %u (%s)",
+	   GET_BE_U_4(sflow_gen_counter->ifindex),
+	   GET_BE_U_4(sflow_gen_counter->iftype),
+	   GET_BE_U_8(sflow_gen_counter->ifspeed),
+	   GET_BE_U_4(sflow_gen_counter->ifdirection),
 	   tok2str(sflow_iface_direction_values, "Unknown",
-	   EXTRACT_32BITS(sflow_gen_counter->ifdirection))));
-    ND_PRINT((ndo, "\n\t      ifstatus %u, adminstatus: %s, operstatus: %s",
-	   EXTRACT_32BITS(sflow_gen_counter->ifstatus),
-	   EXTRACT_32BITS(sflow_gen_counter->ifstatus)&1 ? "up" : "down",
-	   (EXTRACT_32BITS(sflow_gen_counter->ifstatus)>>1)&1 ? "up" : "down"));
-    ND_PRINT((ndo, "\n\t      In octets %" PRIu64
+	   GET_BE_U_4(sflow_gen_counter->ifdirection)));
+    ND_PRINT("\n\t      ifstatus %u, adminstatus: %s, operstatus: %s",
+	   GET_BE_U_4(sflow_gen_counter->ifstatus),
+	   GET_BE_U_4(sflow_gen_counter->ifstatus)&1 ? "up" : "down",
+	   (GET_BE_U_4(sflow_gen_counter->ifstatus)>>1)&1 ? "up" : "down");
+    ND_PRINT("\n\t      In octets %" PRIu64
 	   ", unicast pkts %u, multicast pkts %u, broadcast pkts %u, discards %u",
-	   EXTRACT_64BITS(sflow_gen_counter->ifinoctets),
-	   EXTRACT_32BITS(sflow_gen_counter->ifinunicastpkts),
-	   EXTRACT_32BITS(sflow_gen_counter->ifinmulticastpkts),
-	   EXTRACT_32BITS(sflow_gen_counter->ifinbroadcastpkts),
-	   EXTRACT_32BITS(sflow_gen_counter->ifindiscards)));
-    ND_PRINT((ndo, "\n\t      In errors %u, unknown protos %u",
-	   EXTRACT_32BITS(sflow_gen_counter->ifinerrors),
-	   EXTRACT_32BITS(sflow_gen_counter->ifinunkownprotos)));
-    ND_PRINT((ndo, "\n\t      Out octets %" PRIu64
+	   GET_BE_U_8(sflow_gen_counter->ifinoctets),
+	   GET_BE_U_4(sflow_gen_counter->ifinunicastpkts),
+	   GET_BE_U_4(sflow_gen_counter->ifinmulticastpkts),
+	   GET_BE_U_4(sflow_gen_counter->ifinbroadcastpkts),
+	   GET_BE_U_4(sflow_gen_counter->ifindiscards));
+    ND_PRINT("\n\t      In errors %u, unknown protos %u",
+	   GET_BE_U_4(sflow_gen_counter->ifinerrors),
+	   GET_BE_U_4(sflow_gen_counter->ifinunkownprotos));
+    ND_PRINT("\n\t      Out octets %" PRIu64
 	   ", unicast pkts %u, multicast pkts %u, broadcast pkts %u, discards %u",
-	   EXTRACT_64BITS(sflow_gen_counter->ifoutoctets),
-	   EXTRACT_32BITS(sflow_gen_counter->ifoutunicastpkts),
-	   EXTRACT_32BITS(sflow_gen_counter->ifoutmulticastpkts),
-	   EXTRACT_32BITS(sflow_gen_counter->ifoutbroadcastpkts),
-	   EXTRACT_32BITS(sflow_gen_counter->ifoutdiscards)));
-    ND_PRINT((ndo, "\n\t      Out errors %u, promisc mode %u",
-	   EXTRACT_32BITS(sflow_gen_counter->ifouterrors),
-	   EXTRACT_32BITS(sflow_gen_counter->ifpromiscmode)));
+	   GET_BE_U_8(sflow_gen_counter->ifoutoctets),
+	   GET_BE_U_4(sflow_gen_counter->ifoutunicastpkts),
+	   GET_BE_U_4(sflow_gen_counter->ifoutmulticastpkts),
+	   GET_BE_U_4(sflow_gen_counter->ifoutbroadcastpkts),
+	   GET_BE_U_4(sflow_gen_counter->ifoutdiscards));
+    ND_PRINT("\n\t      Out errors %u, promisc mode %u",
+	   GET_BE_U_4(sflow_gen_counter->ifouterrors),
+	   GET_BE_U_4(sflow_gen_counter->ifpromiscmode));
 
     return 0;
-
-trunc:
-    return 1;
 }
 
 static int
@@ -355,28 +362,24 @@ print_sflow_counter_ethernet(netdissect_options *ndo,
 	return 1;
 
     sflow_eth_counter = (const struct sflow_ethernet_counter_t *)pointer;
-    ND_TCHECK(*sflow_eth_counter);
-    ND_PRINT((ndo, "\n\t      align errors %u, fcs errors %u, single collision %u, multiple collision %u, test error %u",
-	   EXTRACT_32BITS(sflow_eth_counter->alignerrors),
-	   EXTRACT_32BITS(sflow_eth_counter->fcserrors),
-	   EXTRACT_32BITS(sflow_eth_counter->single_collision_frames),
-	   EXTRACT_32BITS(sflow_eth_counter->multiple_collision_frames),
-	   EXTRACT_32BITS(sflow_eth_counter->test_errors)));
-    ND_PRINT((ndo, "\n\t      deferred %u, late collision %u, excessive collision %u, mac trans error %u",
-	   EXTRACT_32BITS(sflow_eth_counter->deferred_transmissions),
-	   EXTRACT_32BITS(sflow_eth_counter->late_collisions),
-	   EXTRACT_32BITS(sflow_eth_counter->excessive_collisions),
-	   EXTRACT_32BITS(sflow_eth_counter->mac_transmit_errors)));
-    ND_PRINT((ndo, "\n\t      carrier error %u, frames too long %u, mac receive errors %u, symbol errors %u",
-	   EXTRACT_32BITS(sflow_eth_counter->carrier_sense_errors),
-	   EXTRACT_32BITS(sflow_eth_counter->frame_too_longs),
-	   EXTRACT_32BITS(sflow_eth_counter->mac_receive_errors),
-	   EXTRACT_32BITS(sflow_eth_counter->symbol_errors)));
+    ND_PRINT("\n\t      align errors %u, fcs errors %u, single collision %u, multiple collision %u, test error %u",
+	   GET_BE_U_4(sflow_eth_counter->alignerrors),
+	   GET_BE_U_4(sflow_eth_counter->fcserrors),
+	   GET_BE_U_4(sflow_eth_counter->single_collision_frames),
+	   GET_BE_U_4(sflow_eth_counter->multiple_collision_frames),
+	   GET_BE_U_4(sflow_eth_counter->test_errors));
+    ND_PRINT("\n\t      deferred %u, late collision %u, excessive collision %u, mac trans error %u",
+	   GET_BE_U_4(sflow_eth_counter->deferred_transmissions),
+	   GET_BE_U_4(sflow_eth_counter->late_collisions),
+	   GET_BE_U_4(sflow_eth_counter->excessive_collisions),
+	   GET_BE_U_4(sflow_eth_counter->mac_transmit_errors));
+    ND_PRINT("\n\t      carrier error %u, frames too long %u, mac receive errors %u, symbol errors %u",
+	   GET_BE_U_4(sflow_eth_counter->carrier_sense_errors),
+	   GET_BE_U_4(sflow_eth_counter->frame_too_longs),
+	   GET_BE_U_4(sflow_eth_counter->mac_receive_errors),
+	   GET_BE_U_4(sflow_eth_counter->symbol_errors));
 
     return 0;
-
-trunc:
-    return 1;
 }
 
 static int
@@ -396,34 +399,30 @@ print_sflow_counter_basevg(netdissect_options *ndo,
 	return 1;
 
     sflow_100basevg_counter = (const struct sflow_100basevg_counter_t *)pointer;
-    ND_TCHECK(*sflow_100basevg_counter);
-    ND_PRINT((ndo, "\n\t      in high prio frames %u, in high prio octets %" PRIu64,
-	   EXTRACT_32BITS(sflow_100basevg_counter->in_highpriority_frames),
-	   EXTRACT_64BITS(sflow_100basevg_counter->in_highpriority_octets)));
-    ND_PRINT((ndo, "\n\t      in norm prio frames %u, in norm prio octets %" PRIu64,
-	   EXTRACT_32BITS(sflow_100basevg_counter->in_normpriority_frames),
-	   EXTRACT_64BITS(sflow_100basevg_counter->in_normpriority_octets)));
-    ND_PRINT((ndo, "\n\t      in ipm errors %u, oversized %u, in data errors %u, null addressed frames %u",
-	   EXTRACT_32BITS(sflow_100basevg_counter->in_ipmerrors),
-	   EXTRACT_32BITS(sflow_100basevg_counter->in_oversized),
-	   EXTRACT_32BITS(sflow_100basevg_counter->in_data_errors),
-	   EXTRACT_32BITS(sflow_100basevg_counter->in_null_addressed_frames)));
-    ND_PRINT((ndo, "\n\t      out high prio frames %u, out high prio octets %" PRIu64
+    ND_PRINT("\n\t      in high prio frames %u, in high prio octets %" PRIu64,
+	   GET_BE_U_4(sflow_100basevg_counter->in_highpriority_frames),
+	   GET_BE_U_8(sflow_100basevg_counter->in_highpriority_octets));
+    ND_PRINT("\n\t      in norm prio frames %u, in norm prio octets %" PRIu64,
+	   GET_BE_U_4(sflow_100basevg_counter->in_normpriority_frames),
+	   GET_BE_U_8(sflow_100basevg_counter->in_normpriority_octets));
+    ND_PRINT("\n\t      in ipm errors %u, oversized %u, in data errors %u, null addressed frames %u",
+	   GET_BE_U_4(sflow_100basevg_counter->in_ipmerrors),
+	   GET_BE_U_4(sflow_100basevg_counter->in_oversized),
+	   GET_BE_U_4(sflow_100basevg_counter->in_data_errors),
+	   GET_BE_U_4(sflow_100basevg_counter->in_null_addressed_frames));
+    ND_PRINT("\n\t      out high prio frames %u, out high prio octets %" PRIu64
 	   ", trans into frames %u",
-	   EXTRACT_32BITS(sflow_100basevg_counter->out_highpriority_frames),
-	   EXTRACT_64BITS(sflow_100basevg_counter->out_highpriority_octets),
-	   EXTRACT_32BITS(sflow_100basevg_counter->transitioninto_frames)));
-    ND_PRINT((ndo, "\n\t      in hc high prio octets %" PRIu64
+	   GET_BE_U_4(sflow_100basevg_counter->out_highpriority_frames),
+	   GET_BE_U_8(sflow_100basevg_counter->out_highpriority_octets),
+	   GET_BE_U_4(sflow_100basevg_counter->transitioninto_frames));
+    ND_PRINT("\n\t      in hc high prio octets %" PRIu64
 	   ", in hc norm prio octets %" PRIu64
 	   ", out hc high prio octets %" PRIu64,
-	   EXTRACT_64BITS(sflow_100basevg_counter->hc_in_highpriority_octets),
-	   EXTRACT_64BITS(sflow_100basevg_counter->hc_in_normpriority_octets),
-	   EXTRACT_64BITS(sflow_100basevg_counter->hc_out_highpriority_octets)));
+	   GET_BE_U_8(sflow_100basevg_counter->hc_in_highpriority_octets),
+	   GET_BE_U_8(sflow_100basevg_counter->hc_in_normpriority_octets),
+	   GET_BE_U_8(sflow_100basevg_counter->hc_out_highpriority_octets));
 
     return 0;
-
-trunc:
-    return 1;
 }
 
 static int
@@ -436,28 +435,24 @@ print_sflow_counter_vlan(netdissect_options *ndo,
 	return 1;
 
     sflow_vlan_counter = (const struct sflow_vlan_counter_t *)pointer;
-    ND_TCHECK(*sflow_vlan_counter);
-    ND_PRINT((ndo, "\n\t      vlan_id %u, octets %" PRIu64
+    ND_PRINT("\n\t      vlan_id %u, octets %" PRIu64
 	   ", unicast_pkt %u, multicast_pkt %u, broadcast_pkt %u, discards %u",
-	   EXTRACT_32BITS(sflow_vlan_counter->vlan_id),
-	   EXTRACT_64BITS(sflow_vlan_counter->octets),
-	   EXTRACT_32BITS(sflow_vlan_counter->unicast_pkt),
-	   EXTRACT_32BITS(sflow_vlan_counter->multicast_pkt),
-	   EXTRACT_32BITS(sflow_vlan_counter->broadcast_pkt),
-	   EXTRACT_32BITS(sflow_vlan_counter->discards)));
+	   GET_BE_U_4(sflow_vlan_counter->vlan_id),
+	   GET_BE_U_8(sflow_vlan_counter->octets),
+	   GET_BE_U_4(sflow_vlan_counter->unicast_pkt),
+	   GET_BE_U_4(sflow_vlan_counter->multicast_pkt),
+	   GET_BE_U_4(sflow_vlan_counter->broadcast_pkt),
+	   GET_BE_U_4(sflow_vlan_counter->discards));
 
     return 0;
-
-trunc:
-    return 1;
 }
 
 struct sflow_processor_counter_t {
-    uint8_t five_sec_util[4];
-    uint8_t one_min_util[4];
-    uint8_t five_min_util[4];
-    uint8_t total_memory[8];
-    uint8_t free_memory[8];
+    nd_uint32_t five_sec_util;
+    nd_uint32_t one_min_util;
+    nd_uint32_t five_min_util;
+    nd_uint64_t total_memory;
+    nd_uint64_t free_memory;
 };
 
 static int
@@ -470,19 +465,15 @@ print_sflow_counter_processor(netdissect_options *ndo,
 	return 1;
 
     sflow_processor_counter = (const struct sflow_processor_counter_t *)pointer;
-    ND_TCHECK(*sflow_processor_counter);
-    ND_PRINT((ndo, "\n\t      5sec %u, 1min %u, 5min %u, total_mem %" PRIu64
+    ND_PRINT("\n\t      5sec %u, 1min %u, 5min %u, total_mem %" PRIu64
 	   ", total_mem %" PRIu64,
-	   EXTRACT_32BITS(sflow_processor_counter->five_sec_util),
-	   EXTRACT_32BITS(sflow_processor_counter->one_min_util),
-	   EXTRACT_32BITS(sflow_processor_counter->five_min_util),
-	   EXTRACT_64BITS(sflow_processor_counter->total_memory),
-	   EXTRACT_64BITS(sflow_processor_counter->free_memory)));
+	   GET_BE_U_4(sflow_processor_counter->five_sec_util),
+	   GET_BE_U_4(sflow_processor_counter->one_min_util),
+	   GET_BE_U_4(sflow_processor_counter->five_min_util),
+	   GET_BE_U_8(sflow_processor_counter->total_memory),
+	   GET_BE_U_8(sflow_processor_counter->free_memory));
 
     return 0;
-
-trunc:
-    return 1;
 }
 
 static int
@@ -506,17 +497,16 @@ sflow_print_counter_records(netdissect_options *ndo,
 	if (tlen < sizeof(struct sflow_counter_record_t))
 	    return 1;
 	sflow_counter_record = (const struct sflow_counter_record_t *)tptr;
-	ND_TCHECK(*sflow_counter_record);
 
-	enterprise = EXTRACT_32BITS(sflow_counter_record->format);
+	enterprise = GET_BE_U_4(sflow_counter_record->format);
 	counter_type = enterprise & 0x0FFF;
 	enterprise = enterprise >> 20;
-	counter_len  = EXTRACT_32BITS(sflow_counter_record->length);
-	ND_PRINT((ndo, "\n\t    enterprise %u, %s (%u) length %u",
+	counter_len  = GET_BE_U_4(sflow_counter_record->length);
+	ND_PRINT("\n\t    enterprise %u, %s (%u) length %u",
 	       enterprise,
 	       (enterprise == 0) ? tok2str(sflow_counter_type_values,"Unknown",counter_type) : "Unknown",
 	       counter_type,
-	       counter_len));
+	       counter_len);
 
 	tptr += sizeof(struct sflow_counter_record_t);
 	tlen -= sizeof(struct sflow_counter_record_t);
@@ -562,9 +552,6 @@ sflow_print_counter_records(netdissect_options *ndo,
     }
 
     return 0;
-
-trunc:
-    return 1;
 }
 
 static int
@@ -573,33 +560,23 @@ sflow_print_counter_sample(netdissect_options *ndo,
 {
     const struct sflow_counter_sample_t *sflow_counter_sample;
     u_int           nrecords;
-    u_int           typesource;
-    u_int           type;
-    u_int           index;
 
     if (len < sizeof(struct sflow_counter_sample_t))
 	return 1;
 
     sflow_counter_sample = (const struct sflow_counter_sample_t *)pointer;
-    ND_TCHECK(*sflow_counter_sample);
 
-    typesource = EXTRACT_32BITS(sflow_counter_sample->typesource);
-    nrecords   = EXTRACT_32BITS(sflow_counter_sample->records);
-    type = typesource >> 24;
-    index = typesource & 0x0FFF;
+    nrecords   = GET_BE_U_4(sflow_counter_sample->records);
 
-    ND_PRINT((ndo, " seqnum %u, type %u, idx %u, records %u",
-	   EXTRACT_32BITS(sflow_counter_sample->seqnum),
-	   type,
-	   index,
-	   nrecords));
+    ND_PRINT(" seqnum %u, type %u, idx %u, records %u",
+	   GET_BE_U_4(sflow_counter_sample->seqnum),
+	   GET_U_1(sflow_counter_sample->type),
+	   GET_BE_U_3(sflow_counter_sample->index),
+	   nrecords);
 
     return sflow_print_counter_records(ndo, pointer + sizeof(struct sflow_counter_sample_t),
 				       len - sizeof(struct sflow_counter_sample_t),
 				       nrecords);
-
-trunc:
-    return 1;
 }
 
 static int
@@ -614,22 +591,18 @@ sflow_print_expanded_counter_sample(netdissect_options *ndo,
 	return 1;
 
     sflow_expanded_counter_sample = (const struct sflow_expanded_counter_sample_t *)pointer;
-    ND_TCHECK(*sflow_expanded_counter_sample);
 
-    nrecords = EXTRACT_32BITS(sflow_expanded_counter_sample->records);
+    nrecords = GET_BE_U_4(sflow_expanded_counter_sample->records);
 
-    ND_PRINT((ndo, " seqnum %u, type %u, idx %u, records %u",
-	   EXTRACT_32BITS(sflow_expanded_counter_sample->seqnum),
-	   EXTRACT_32BITS(sflow_expanded_counter_sample->type),
-	   EXTRACT_32BITS(sflow_expanded_counter_sample->index),
-	   nrecords));
+    ND_PRINT(" seqnum %u, type %u, idx %u, records %u",
+	   GET_BE_U_4(sflow_expanded_counter_sample->seqnum),
+	   GET_BE_U_4(sflow_expanded_counter_sample->type),
+	   GET_BE_U_4(sflow_expanded_counter_sample->index),
+	   nrecords);
 
     return sflow_print_counter_records(ndo, pointer + sizeof(struct sflow_expanded_counter_sample_t),
 				       len - sizeof(struct sflow_expanded_counter_sample_t),
 				       nrecords);
-
-trunc:
-    return 1;
 }
 
 static int
@@ -642,21 +615,17 @@ print_sflow_raw_packet(netdissect_options *ndo,
 	return 1;
 
     sflow_flow_raw = (const struct sflow_expanded_flow_raw_t *)pointer;
-    ND_TCHECK(*sflow_flow_raw);
-    ND_PRINT((ndo, "\n\t      protocol %s (%u), length %u, stripped bytes %u, header_size %u",
-	   tok2str(sflow_flow_raw_protocol_values,"Unknown",EXTRACT_32BITS(sflow_flow_raw->protocol)),
-	   EXTRACT_32BITS(sflow_flow_raw->protocol),
-	   EXTRACT_32BITS(sflow_flow_raw->length),
-	   EXTRACT_32BITS(sflow_flow_raw->stripped_bytes),
-	   EXTRACT_32BITS(sflow_flow_raw->header_size)));
+    ND_PRINT("\n\t      protocol %s (%u), length %u, stripped bytes %u, header_size %u",
+	   tok2str(sflow_flow_raw_protocol_values,"Unknown",GET_BE_U_4(sflow_flow_raw->protocol)),
+	   GET_BE_U_4(sflow_flow_raw->protocol),
+	   GET_BE_U_4(sflow_flow_raw->length),
+	   GET_BE_U_4(sflow_flow_raw->stripped_bytes),
+	   GET_BE_U_4(sflow_flow_raw->header_size));
 
     /* QUESTION - should we attempt to print the raw header itself?
-       assuming of course there is wnough data present to do so... */
+       assuming of course there is enough data present to do so... */
 
     return 0;
-
-trunc:
-    return 1;
 }
 
 static int
@@ -669,16 +638,12 @@ print_sflow_ethernet_frame(netdissect_options *ndo,
 	return 1;
 
     sflow_ethernet_frame = (const struct sflow_ethernet_frame_t *)pointer;
-    ND_TCHECK(*sflow_ethernet_frame);
 
-    ND_PRINT((ndo, "\n\t      frame len %u, type %u",
-	   EXTRACT_32BITS(sflow_ethernet_frame->length),
-	   EXTRACT_32BITS(sflow_ethernet_frame->type)));
+    ND_PRINT("\n\t      frame len %u, type %u",
+	   GET_BE_U_4(sflow_ethernet_frame->length),
+	   GET_BE_U_4(sflow_ethernet_frame->type));
 
     return 0;
-
-trunc:
-    return 1;
 }
 
 static int
@@ -691,17 +656,13 @@ print_sflow_extended_switch_data(netdissect_options *ndo,
 	return 1;
 
     sflow_extended_sw_data = (const struct sflow_extended_switch_data_t *)pointer;
-    ND_TCHECK(*sflow_extended_sw_data);
-    ND_PRINT((ndo, "\n\t      src vlan %u, src pri %u, dst vlan %u, dst pri %u",
-	   EXTRACT_32BITS(sflow_extended_sw_data->src_vlan),
-	   EXTRACT_32BITS(sflow_extended_sw_data->src_pri),
-	   EXTRACT_32BITS(sflow_extended_sw_data->dst_vlan),
-	   EXTRACT_32BITS(sflow_extended_sw_data->dst_pri)));
+    ND_PRINT("\n\t      src vlan %u, src pri %u, dst vlan %u, dst pri %u",
+	   GET_BE_U_4(sflow_extended_sw_data->src_vlan),
+	   GET_BE_U_4(sflow_extended_sw_data->src_pri),
+	   GET_BE_U_4(sflow_extended_sw_data->dst_vlan),
+	   GET_BE_U_4(sflow_extended_sw_data->dst_pri));
 
     return 0;
-
-trunc:
-    return 1;
 }
 
 static int
@@ -726,20 +687,19 @@ sflow_print_flow_records(netdissect_options *ndo,
 	    return 1;
 
 	sflow_flow_record = (const struct sflow_flow_record_t *)tptr;
-	ND_TCHECK(*sflow_flow_record);
 
-	/* so, the funky encoding means we cannot blythly mask-off
+	/* so, the funky encoding means we cannot blithely mask-off
 	   bits, we must also check the enterprise. */
 
-	enterprise = EXTRACT_32BITS(sflow_flow_record->format);
+	enterprise = GET_BE_U_4(sflow_flow_record->format);
 	flow_type = enterprise & 0x0FFF;
 	enterprise = enterprise >> 12;
-	flow_len  = EXTRACT_32BITS(sflow_flow_record->length);
-	ND_PRINT((ndo, "\n\t    enterprise %u %s (%u) length %u",
+	flow_len  = GET_BE_U_4(sflow_flow_record->length);
+	ND_PRINT("\n\t    enterprise %u %s (%u) length %u",
 	       enterprise,
 	       (enterprise == 0) ? tok2str(sflow_flow_type_values,"Unknown",flow_type) : "Unknown",
 	       flow_type,
-	       flow_len));
+	       flow_len);
 
 	tptr += sizeof(struct sflow_flow_record_t);
 	tlen -= sizeof(struct sflow_flow_record_t);
@@ -789,9 +749,6 @@ sflow_print_flow_records(netdissect_options *ndo,
     }
 
     return 0;
-
-trunc:
-    return 1;
 }
 
 static int
@@ -800,38 +757,28 @@ sflow_print_flow_sample(netdissect_options *ndo,
 {
     const struct sflow_flow_sample_t *sflow_flow_sample;
     u_int          nrecords;
-    u_int          typesource;
-    u_int          type;
-    u_int          index;
 
     if (len < sizeof(struct sflow_flow_sample_t))
 	return 1;
 
     sflow_flow_sample = (const struct sflow_flow_sample_t *)pointer;
-    ND_TCHECK(*sflow_flow_sample);
 
-    typesource = EXTRACT_32BITS(sflow_flow_sample->typesource);
-    nrecords = EXTRACT_32BITS(sflow_flow_sample->records);
-    type = typesource >> 24;
-    index = typesource & 0x0FFF;
+    nrecords = GET_BE_U_4(sflow_flow_sample->records);
 
-    ND_PRINT((ndo, " seqnum %u, type %u, idx %u, rate %u, pool %u, drops %u, input %u output %u records %u",
-	   EXTRACT_32BITS(sflow_flow_sample->seqnum),
-	   type,
-	   index,
-	   EXTRACT_32BITS(sflow_flow_sample->rate),
-	   EXTRACT_32BITS(sflow_flow_sample->pool),
-	   EXTRACT_32BITS(sflow_flow_sample->drops),
-	   EXTRACT_32BITS(sflow_flow_sample->in_interface),
-	   EXTRACT_32BITS(sflow_flow_sample->out_interface),
-	   nrecords));
+    ND_PRINT(" seqnum %u, type %u, idx %u, rate %u, pool %u, drops %u, input %u output %u records %u",
+	   GET_BE_U_4(sflow_flow_sample->seqnum),
+	   GET_U_1(sflow_flow_sample->type),
+	   GET_BE_U_3(sflow_flow_sample->index),
+	   GET_BE_U_4(sflow_flow_sample->rate),
+	   GET_BE_U_4(sflow_flow_sample->pool),
+	   GET_BE_U_4(sflow_flow_sample->drops),
+	   GET_BE_U_4(sflow_flow_sample->in_interface),
+	   GET_BE_U_4(sflow_flow_sample->out_interface),
+	   nrecords);
 
     return sflow_print_flow_records(ndo, pointer + sizeof(struct sflow_flow_sample_t),
 				    len - sizeof(struct sflow_flow_sample_t),
 				    nrecords);
-
-trunc:
-    return 1;
 }
 
 static int
@@ -845,25 +792,21 @@ sflow_print_expanded_flow_sample(netdissect_options *ndo,
 	return 1;
 
     sflow_expanded_flow_sample = (const struct sflow_expanded_flow_sample_t *)pointer;
-    ND_TCHECK(*sflow_expanded_flow_sample);
 
-    nrecords = EXTRACT_32BITS(sflow_expanded_flow_sample->records);
+    nrecords = GET_BE_U_4(sflow_expanded_flow_sample->records);
 
-    ND_PRINT((ndo, " seqnum %u, type %u, idx %u, rate %u, pool %u, drops %u, records %u",
-	   EXTRACT_32BITS(sflow_expanded_flow_sample->seqnum),
-	   EXTRACT_32BITS(sflow_expanded_flow_sample->type),
-	   EXTRACT_32BITS(sflow_expanded_flow_sample->index),
-	   EXTRACT_32BITS(sflow_expanded_flow_sample->rate),
-	   EXTRACT_32BITS(sflow_expanded_flow_sample->pool),
-	   EXTRACT_32BITS(sflow_expanded_flow_sample->drops),
-	   EXTRACT_32BITS(sflow_expanded_flow_sample->records)));
+    ND_PRINT(" seqnum %u, type %u, idx %u, rate %u, pool %u, drops %u, records %u",
+	   GET_BE_U_4(sflow_expanded_flow_sample->seqnum),
+	   GET_BE_U_4(sflow_expanded_flow_sample->type),
+	   GET_BE_U_4(sflow_expanded_flow_sample->index),
+	   GET_BE_U_4(sflow_expanded_flow_sample->rate),
+	   GET_BE_U_4(sflow_expanded_flow_sample->pool),
+	   GET_BE_U_4(sflow_expanded_flow_sample->drops),
+	   nrecords);
 
     return sflow_print_flow_records(ndo, pointer + sizeof(struct sflow_expanded_flow_sample_t),
 				    len - sizeof(struct sflow_expanded_flow_sample_t),
 				    nrecords);
-
-trunc:
-    return 1;
 }
 
 void
@@ -871,80 +814,102 @@ sflow_print(netdissect_options *ndo,
             const u_char *pptr, u_int len)
 {
     const struct sflow_datagram_t *sflow_datagram;
+    const struct sflow_v6_datagram_t *sflow_v6_datagram;
     const struct sflow_sample_header *sflow_sample;
 
     const u_char *tptr;
     u_int tlen;
     uint32_t sflow_sample_type, sflow_sample_len;
     uint32_t nsamples;
+    uint32_t ip_version;
 
+    ndo->ndo_protocol = "sflow";
     tptr = pptr;
     tlen = len;
     sflow_datagram = (const struct sflow_datagram_t *)pptr;
-    if (len < sizeof(struct sflow_datagram_t)) {
-        ND_TCHECK(sflow_datagram->version);
-        ND_PRINT((ndo, "sFlowv%u", EXTRACT_32BITS(sflow_datagram->version)));
-        ND_PRINT((ndo, " [length %u < %zu]",
-                  len, sizeof(struct sflow_datagram_t)));
-        ND_PRINT((ndo, " (invalid)"));
+    sflow_v6_datagram = (const struct sflow_v6_datagram_t *)pptr;
+    ip_version = GET_BE_U_4(sflow_datagram->ip_version);
+
+    if ((len < sizeof(struct sflow_datagram_t) && (ip_version == 1)) ||
+        (len < sizeof(struct sflow_v6_datagram_t) && (ip_version == 2))) {
+        ND_PRINT("sFlowv%u", GET_BE_U_4(sflow_datagram->version));
+        ND_PRINT(" [length %u < %zu]", len, sizeof(struct sflow_datagram_t));
+        nd_print_invalid(ndo);
         return;
     }
-    ND_TCHECK(*sflow_datagram);
+    ND_TCHECK_SIZE(sflow_datagram);
 
     /*
      * Sanity checking of the header.
      */
-    if (EXTRACT_32BITS(sflow_datagram->version) != 5) {
-        ND_PRINT((ndo, "sFlow version %u packet not supported",
-               EXTRACT_32BITS(sflow_datagram->version)));
+    if (GET_BE_U_4(sflow_datagram->version) != 5) {
+        ND_PRINT("sFlow version %u packet not supported",
+               GET_BE_U_4(sflow_datagram->version));
         return;
     }
 
     if (ndo->ndo_vflag < 1) {
-        ND_PRINT((ndo, "sFlowv%u, %s agent %s, agent-id %u, length %u",
-               EXTRACT_32BITS(sflow_datagram->version),
-               EXTRACT_32BITS(sflow_datagram->ip_version) == 1 ? "IPv4" : "IPv6",
-               ipaddr_string(ndo, sflow_datagram->agent),
-               EXTRACT_32BITS(sflow_datagram->agent_id),
-               len));
+        ND_PRINT("sFlowv%u, %s agent %s, agent-id %u, length %u",
+               GET_BE_U_4(sflow_datagram->version),
+               ip_version == 1 ? "IPv4" : "IPv6",
+               ip_version == 1 ? GET_IPADDR_STRING(sflow_datagram->agent) :
+                                 GET_IP6ADDR_STRING( sflow_v6_datagram->agent),
+               ip_version == 1 ? GET_BE_U_4(sflow_datagram->agent_id) :
+                                 GET_BE_U_4(sflow_v6_datagram->agent_id),
+               len);
         return;
     }
 
     /* ok they seem to want to know everything - lets fully decode it */
-    nsamples=EXTRACT_32BITS(sflow_datagram->samples);
-    ND_PRINT((ndo, "sFlowv%u, %s agent %s, agent-id %u, seqnum %u, uptime %u, samples %u, length %u",
-           EXTRACT_32BITS(sflow_datagram->version),
-           EXTRACT_32BITS(sflow_datagram->ip_version) == 1 ? "IPv4" : "IPv6",
-           ipaddr_string(ndo, sflow_datagram->agent),
-           EXTRACT_32BITS(sflow_datagram->agent_id),
-           EXTRACT_32BITS(sflow_datagram->seqnum),
-           EXTRACT_32BITS(sflow_datagram->uptime),
-           nsamples,
-           len));
+    if (ip_version == 1) {
+        nsamples=GET_BE_U_4(sflow_datagram->samples);
+        ND_PRINT("sFlowv%u, %s agent %s, agent-id %u, seqnum %u, uptime %u, samples %u, length %u",
+               GET_BE_U_4(sflow_datagram->version),
+               "IPv4",
+               GET_IPADDR_STRING(sflow_datagram->agent),
+               GET_BE_U_4(sflow_datagram->agent_id),
+               GET_BE_U_4(sflow_datagram->seqnum),
+               GET_BE_U_4(sflow_datagram->uptime),
+               nsamples,
+               len);
 
-    /* skip Common header */
-    tptr += sizeof(const struct sflow_datagram_t);
-
-    if(tlen <= sizeof(const struct sflow_datagram_t)) goto trunc;
-    tlen -= sizeof(const struct sflow_datagram_t);
+        /* skip Common header */
+        ND_ICHECK_ZU(tlen, <, sizeof(struct sflow_datagram_t));
+        tptr += sizeof(struct sflow_datagram_t);
+        tlen -= sizeof(struct sflow_datagram_t);
+    } else {
+        nsamples=GET_BE_U_4(sflow_v6_datagram->samples);
+        ND_PRINT("sFlowv%u, %s agent %s, agent-id %u, seqnum %u, uptime %u, samples %u, length %u",
+               GET_BE_U_4(sflow_v6_datagram->version),
+               "IPv6",
+               GET_IP6ADDR_STRING(sflow_v6_datagram->agent),
+               GET_BE_U_4(sflow_v6_datagram->agent_id),
+               GET_BE_U_4(sflow_v6_datagram->seqnum),
+               GET_BE_U_4(sflow_v6_datagram->uptime),
+               nsamples,
+               len);
 
+        /* skip Common header */
+        ND_ICHECK_ZU(tlen, <, sizeof(struct sflow_v6_datagram_t));
+        tptr += sizeof(struct sflow_v6_datagram_t);
+        tlen -= sizeof(struct sflow_v6_datagram_t);
+    }
     while (nsamples > 0 && tlen > 0) {
         sflow_sample = (const struct sflow_sample_header *)tptr;
-        ND_TCHECK(*sflow_sample);
 
-        sflow_sample_type = (EXTRACT_32BITS(sflow_sample->format)&0x0FFF);
-        sflow_sample_len = EXTRACT_32BITS(sflow_sample->len);
+        sflow_sample_type = (GET_BE_U_4(sflow_sample->format)&0x0FFF);
+        sflow_sample_len = GET_BE_U_4(sflow_sample->len);
 
 	if (tlen < sizeof(struct sflow_sample_header))
-	    goto trunc;
+	    goto invalid;
 
         tptr += sizeof(struct sflow_sample_header);
         tlen -= sizeof(struct sflow_sample_header);
 
-        ND_PRINT((ndo, "\n\t%s (%u), length %u,",
+        ND_PRINT("\n\t%s (%u), length %u,",
                tok2str(sflow_format_values, "Unknown", sflow_sample_type),
                sflow_sample_type,
-               sflow_sample_len));
+               sflow_sample_len);
 
         /* basic sanity check */
         if (sflow_sample_type == 0 || sflow_sample_len ==0) {
@@ -952,30 +917,30 @@ sflow_print(netdissect_options *ndo,
         }
 
 	if (tlen < sflow_sample_len)
-	    goto trunc;
+	    goto invalid;
 
         /* did we capture enough for fully decoding the sample ? */
-        ND_TCHECK2(*tptr, sflow_sample_len);
+        ND_TCHECK_LEN(tptr, sflow_sample_len);
 
 	switch(sflow_sample_type) {
         case SFLOW_FLOW_SAMPLE:
 	    if (sflow_print_flow_sample(ndo, tptr, tlen))
-		goto trunc;
+		goto invalid;
             break;
 
         case SFLOW_COUNTER_SAMPLE:
 	    if (sflow_print_counter_sample(ndo, tptr,tlen))
-		goto trunc;
+		goto invalid;
             break;
 
         case SFLOW_EXPANDED_FLOW_SAMPLE:
 	    if (sflow_print_expanded_flow_sample(ndo, tptr, tlen))
-		goto trunc;
+		goto invalid;
 	    break;
 
         case SFLOW_EXPANDED_COUNTER_SAMPLE:
 	    if (sflow_print_expanded_counter_sample(ndo, tptr,tlen))
-		goto trunc;
+		goto invalid;
 	    break;
 
         default:
@@ -989,13 +954,7 @@ sflow_print(netdissect_options *ndo,
     }
     return;
 
- trunc:
-    ND_PRINT((ndo, "[|SFLOW]"));
+invalid:
+    nd_print_invalid(ndo);
+    ND_TCHECK_LEN(tptr, tlen);
 }
-
-/*
- * Local Variables:
- * c-style: whitesmith
- * c-basic-offset: 4
- * End:
- */
